
# Folder and File Structure

## Pi ##

This directory contains the scripts that will be run on the Raspberry Pi.
This includes individual Python scripts for each part of functionality e.g. `run_inference.py`, along with the `main.py` file that contains the main Python workflow that is run as part of the deployment from Node-Red.
This directory also contains the `start.sh` script that runs the Node-Red deployment with the correct parameters.

### Physical Folder ###

The Physical folder is the base for all sensing and actuating functions. It holds the two folders - Arduino and Node-Red, as well as the file lcd_display.py, which is used by the Node-Red for LCD outputs.

The Arduino folder holds the two Arduino sketches, one for each Arduino (Arduino 0 and Arduino 1). These Arduino numbers are to be assigned based on the ACM port each Arduino is connected to via USB (ACM0 and ACM1). The Node-Red folder holds the Node-Red flow .json which needs to be manually imported to Node-Red.

### Other Folders ###

Other folders in the Pi directory act as logical separation for scripts within the main Python workflow.

# Pi Setup Instructions #
### Prerequisites ###
**Python and Venv**
- Install Python version 3.10.9
- Ensure that you have the correct version of python selected with `python --version`
  - If the response is not `Python Version 3.10.9` or similar, change your active version to the newly installed version
- Navigate to the `pi` directory with `cd cm2305/pi`
- Create the virtual environment with `python -m venv ecosort`
- Activate the environment with `./ecosort/bin/activate` (Linux) or `./ecosort/Scripts/activate` (Windows)
- install all required libraries with `pip install -r ./requirements.txt`

**Node-Red**
- Install Node-Red with `bash <(curl -sL https://raw.githubusercontent.com/node-red/linux-installers/master/deb/update-nodejs-and-nodered)`.
- On completion, run it with `node-red`. Open the given link for the browser interface.
- Navigate to *Settings>Manage Palette*.
- Ensure the following packages appear:
    - `node-red`
    - `node-red-node-serialport`
- If not, search for and install them.

**Arduino IDE**
- Install the IDE with `sudo apt-get install arduino`.
- With the IDE, go to *Tools>Manage Libraries...*.
- Install the `Servo` library.

### Installation and Configuration ###
1. ***Set Up Physical Components:***
   - **1x** Raspberry Pi
   - **1x** Raspberry Pi GrovePi+ Board
   - **2x** Arduino
   - **2x** Arduino Grove Hat
   - **2x** Servo Motor
   - **2x** Button
   - **1x** Motion Sensor
   - **1x** Pi Camera Module
   - **2x** USB-A to MicroUSB Cable
   - **1x** V5.0 RGB LCD Screen

- **Raspberry Pi:**
  - Install **GrovePi+ Board**
  - **Pin I2C (any):** V5.0 RGB LCD Screen
  - **Ribbon Cable Port:** Pi Camera Module

- **Arduino 0:**
  - Install **Grove Hat**
  - **Pin D5:** Servo Motor  
  - **Pin D3:** Button  
  - **Pin D6:** Button  
  - Connect to Raspberry Pi using MicroUSB cable (ACM Port 0)

- **Arduino 1:**
  - Install **Grove Hat**
  - **Pin D5:** Servo Motor
  - **Pin D2:** Motion Sensor
  - Connect to Raspberry Pi using MicroUSB cable (ACM Port 1)

2. ***Upload Arduino Sketches with Arduino IDE***
  - Arduino 0: `arduino0_motor_button`
  - Arduino 1: `arduino1_motor_motionsensor`

3. ***Import Node-Red Flow from `cm2305/pi/physical/node-red` by Clicking ****Import**** in Node-Red Browser Interface***


# Running the Project #
- On the Pi, run `~/cm2305/pi/start.sh`. On running, this should start Node-Red with required libraries.
# Third-Party Software and Frameworks #

- **Python 3.10.9**
  - Purpose: Core programming language used for backend scripts and control logic on the Raspberry Pi.
  - Documentation: [https://www.python.org/downloads/release/python-3109/](https://www.python.org/downloads/release/python-3109/)

- **OpenCV**
  - Purpose: Python library for image processing and computer vision, intregal to classification pipeline.
  - Documentation: [https://pypi.org/project/opencv-python/](https://pypi.org/project/opencv-python/)

- **Inference SDK**
  - Purpose: Python library to integrate RoboFlow workflows, used for running CV model.
  - Documentation: [https://pypi.org/project/inference-sdk/](https://pypi.org/project/inference-sdk/)

- **Node-RED** (latest via official installer)
  - Purpose: Visual programming environment to wire together hardware, APIs, and data flows.
  - Documentation: [https://nodered.org/docs/](https://nodered.org/docs/)

- **node-red-node-serialport** (latest)
  - Purpose: Node-RED node to enable serial communication with Arduino boards over USB.
  - Documentation: [https://flows.nodered.org/node/node-red-node-serialport](https://flows.nodered.org/node/node-red-node-serialport)

- **Arduino IDE 1.8.xx or newer**
  - Purpose: Used to write and upload sketches to the two Arduinos.
  - Documentation: [https://www.arduino.cc/en/software](https://www.arduino.cc/en/software)

- **Servo Arduino Library 1.1.8**
  - Purpose: Controls servo motors connected to Arduino pins.
  - Documentation: [https://www.arduino.cc/reference/en/libraries/servo/](https://www.arduino.cc/reference/en/libraries/servo/)

- **Node.js 22.14.0 and npm**
  - Purpose: JavaScript runtime and package manager required to install and run Node-RED on the Raspberry Pi.
  - Documentation: [https://nodejs.org/en](https://nodejs.org/en)

- **smbus2 0.5.0**
  - Purpose: Python library used for I2C communication with devices like the Grove LCD screen.
  - Documentation: [https://pypi.org/project/smbus2/](https://pypi.org/project/smbus2/)

- **RPi.GPIO 0.7.1**
  - Purpose: Python library for controlling GPIO pins on the Raspberry Pi.
  - Documentation: [https://pypi.org/project/RPi.GPIO/](https://pypi.org/project/RPi.GPIO/)
 
  ***See `requirements.txt` in Pi folder for more details on onboard libraries - some aren't covered here***

# Code Documentation #
View `pi/requirements.txt` for more details on onboard libraries.


Code is primarily documented via inline annotations.


The Node-Red flow features two 'comment' nodes that act as pointers for developers, such as an explaination of the Lock and Unlock nodes.
# Troubleshooting #
### Pi ###
#### Swapped Arduino Functions ####
It's important that the two Arduinos are correctly configured with the right components, sketch, and serial port. If, on monitoring in Node-Red, the Arduinos have swapped functions, it's most likely the serial ports are incorrect. It should be Arduino 0 = ACM0 and Arduino 1 = ACM1. The easiest way to correct this is by unplugging both Arduinos and replugging them, with Arduino 0 being plugged in first.

#### Stranded in Middle of Node-Red Flow ####
If a component is misconfigured and interferes at a point in time after passing the Lock node but not the Unlock node, the system will never reset and won't take accept new inputs. If this happens, stop Node-Red with ctrl+c and re-enter `~/cm2305/pi/start.sh`.