def parse_result(result):
    # we ignore items larger then 60% of the image size as false positives
    MAX_ITEM_AREA_RATIO = 0.6
    # and we ignore items that are larger than 70% of the image size in at least 1 direction
    MAX_ITEM_LENGTH_RATIO = 0.7
    max_confidence = 0.0

    predictions = result["predictions"]
    image_size = (result["image"]["width"], result["image"]["height"])
    image_area = image_size[0] * image_size[1]

    # if there is no recyclable material detected, classify as non-recyclable
    if not predictions:
        output_type = 'Non-Recyclable'

    for i, item in enumerate(predictions):

        # ignore large items as false positives
        bbox_size = (item["width"], item["height"])
        bbox_area = bbox_size[0] * bbox_size[1]
        if image_area * MAX_ITEM_AREA_RATIO < bbox_area:
            continue
        if image_size[0] * MAX_ITEM_LENGTH_RATIO < bbox_size[0] or image_size[1] * MAX_ITEM_LENGTH_RATIO < bbox_size[1]:
            continue

        # update the max confidence based on current item
        if item['confidence'] > max_confidence:
            max_confidence = item['confidence']
            output_type = item['class']

    if max_confidence > 0.25:
        output_type = item['class']
    else:
        output_type = 'Non-Recyclable'
    return output_type, max_confidence
