// --------------------------------------------------------------------------
// Arduino sketch for controlling a servo motor and reading button presses
// The very core concepts of this sketch were taken from Lab1_code2
// https://gitlab.com/IOTGarage/iot-lab-book/-/tree/070cf41b33cffdc1f1f90f6a42e1e16215db24be/LAB%2001%20-%20Micro-Controller%20Programming/Lab1_code2
// However two-way serial communication was added to allow the servo to be controlled from Node-Red, and the buttons to send messages to Node-Red
// --------------------------------------------------------------------------

// Include the Servo library
#include <Servo.h>

// Define pin for Servo
#define SERVO_PIN 5  // Grove Servo (D5)

// Define pins for Grove LED Buttons
// The grove buttons must be connected to the pin below the Arduino pin number
// They are unique in this case, and don't work like other Grove modules
// For example, if the button is connected to D3, the pin number is 4:
#define BUTTON1_PIN 4  // Grove Button 1 (D3)
#define BUTTON2_PIN 7  // Grove Button 2 (D6)

// Create Servo object
Servo myservo;

void setup() {
  Serial.begin(9600); // Start serial communication at 9600 baud

  // Initialise button pins as input_pullup
  // This means the button will read LOW when pressed and HIGH when not pressed
  // This change was made from the original code, which used INPUT
  // This is because, through research, we found input_pullup is more reliable for buttons
  pinMode(BUTTON1_PIN, INPUT_PULLUP);
  pinMode(BUTTON2_PIN, INPUT_PULLUP);
  
  Serial.println("System Ready");
}

void loop()
{
  // Check for USB serial commands from Node-RED (or any other source)
  if (Serial.available()) {
    String command = Serial.readStringUntil('\n'); // Read full command (commands end with '\n' by default)
    command.trim(); // Remove whitespace

    Serial.println("Received: " + command); // Serial output for debugging

    // Check if the command starts with "SERVO " (e.g. "SERVO 90")
    // This is the expected format for controlling the servo
    if (command.startsWith("SERVO ")) {
      int angle = command.substring(6).toInt(); // Extract angle value

      // Ensure the angle is between 0 and 180
      if (angle >= 0 && angle <= 180) {
        myservo.attach(SERVO_PIN);
        myservo.write(angle); // Move servo to specified angle
        delay(1000); // Wait for the servo to reach the position
        myservo.detach(); // Detach servo to allow free spinning
        Serial.print("Servo moved to: ");
        Serial.println(angle);
      } else {
        // Return an error message if the angle is invalid, this will be sent to Node-RED
        Serial.println("Invalid angle! Must be 0-180.");
      }
    }
  }

  // Check button presses and send messages
  if (digitalRead(BUTTON1_PIN) == LOW) {
    Serial.println("BUTTON1 PRESSED");
    delay(300); // Debounce delay to prevent multiple readings
  }

  if (digitalRead(BUTTON2_PIN) == LOW) {
    Serial.println("BUTTON2 PRESSED");
    delay(300); // Debounce delay to prevent multiple readings
  }
}
