// --------------------------------------------------------------------------
// Arduino sketch for controlling a servo motor and motion sensing
// The very core concepts of this sketch were taken from Lab1_code1
// https://gitlab.com/IOTGarage/iot-lab-book/-/blob/070cf41b33cffdc1f1f90f6a42e1e16215db24be/LAB%2001%20-%20Micro-Controller%20Programming/Lab1_code1
// However two-way serial communication was added to allow the servo to be controlled from Node-Red, and the PIR motion sensor to send messages to Node-Red
// This motion sensor also has a toggle function, allowing it to be turned on and off from Node-Red, allowing for timely servo control when needed
// --------------------------------------------------------------------------


// Include the Servo library
#include <Servo.h>

// Define pins
#define PIR_MOTION_SENSOR_PIN 2
#define SERVO_PIN 5

// Create Servo object
Servo myservo;

// Motion sensing control flag
bool motionSensingEnabled = true;

void setup() {
  Serial.begin(9600); // Start serial communication at 9600 baud

  // Initialise PIR motion sensor pin as input
  pinMode(PIR_MOTION_SENSOR_PIN, INPUT);
}

void loop() {
  // Check for USB commands from Node-RED (or any other source)
  if (Serial.available()) {
    String command = Serial.readStringUntil('\n'); // Read full command (commands end with '\n' by default)
    command.trim(); // Remove whitespace

    Serial.println("Received: " + command); // Serial output for debugging

    // Check if the command starts with "SERVO " (e.g., "SERVO 90")
    // This is the expected format for controlling the servo
    if (command.startsWith("SERVO ")) {
      int angle = command.substring(6).toInt(); // Extract angle value

      // Ensure the angle is between 0 and 180
      if (angle >= 0 && angle <= 180) {
        myservo.attach(SERVO_PIN);
        myservo.write(angle); // Move servo to specified angle
        delay(1000); // Wait for the servo to reach the position
        myservo.detach(); // Detach servo to allow free spinning
        Serial.print("Servo moved to: ");
        Serial.println(angle);
      } else {
        // Return an error message if the angle is invalid, this will be sent to Node-RED
        Serial.println("Invalid angle! Must be 0-180.");
      }
    }

    // Handle motion sensing control commands
    // Check if the command is "STOP SENSING" or "START SENSING"
    // Then enable or disable motion sensing accordingly
    // This allows motion sensing to be turned off from Node-Red
    else if (command == "STOP SENSING") {
      motionSensingEnabled = false;
      Serial.println("Motion sensing disabled.");
    }
    else if (command == "START SENSING") {
      motionSensingEnabled = true;
      Serial.println("Motion sensing enabled.");
    }
  }

  // Check PIR only if sensing is enabled
  if (motionSensingEnabled && digitalRead(PIR_MOTION_SENSOR_PIN)) {
    Serial.println("Movement");
    delay(1000); // Debounce delay to prevent multiple readings
  }
}
